# 15 "src/ocaml/preprocess/lexer_raw.mll"
 
open Std
open Misc
open Lexing
open Parser_raw

type keywords = (string, Parser_raw.token) Hashtbl.t

type error =
  | Illegal_character of char
  | Illegal_escape of string
  | Unterminated_comment of Location.t
  | Unterminated_string
  | Unterminated_string_in_comment of Location.t * Location.t
  | Keyword_as_label of string
  | Literal_overflow of string

exception Error of error * Location.t

(* Monad in which the lexer evaluates *)
type 'a result =
  | Return of 'a
  | Refill of (unit -> 'a result)
  | Fail of error * Location.t

let return a = Return a

let fail e l = Fail (e,l)

let rec (>>=) (m : 'a result) (f : 'a -> 'b result) : 'b result =
  match m with
  | Return a -> f a
  | Refill u ->
    Refill (fun () -> u () >>= f)
  | Fail _ as e -> e

type preprocessor = (Lexing.lexbuf -> Parser_raw.token) -> Lexing.lexbuf -> Parser_raw.token

type state = {
  keywords: keywords;
  mutable buffer: Buffer.t;
  mutable string_start_loc: Location.t;
  mutable comment_start_loc: Location.t list;
  mutable preprocessor: preprocessor option;
}

let make ?preprocessor keywords = {
  keywords;
  buffer = Buffer.create 17;
  string_start_loc = Location.none;
  comment_start_loc = [];
  preprocessor;
}

let lABEL m = m >>= fun v -> return (LABEL v)
let oPTLABEL m = m >>= fun v -> return (OPTLABEL v)

let rec catch m f = match m with
  | Fail (e,l) -> f e l
  | Refill next -> Refill (fun () -> catch (next ()) f)
  | Return _ -> m

(* The table of keywords *)

let keyword_table : keywords =
  create_hashtable 149 [
    "and", AND;
    "as", AS;
    "assert", ASSERT;
    "begin", BEGIN;
    "class", CLASS;
    "constraint", CONSTRAINT;
    "do", DO;
    "done", DONE;
    "downto", DOWNTO;
    "else", ELSE;
    "end", END;
    "exception", EXCEPTION;
    "external", EXTERNAL;
    "false", FALSE;
    "for", FOR;
    "fun", FUN;
    "function", FUNCTION;
    "functor", FUNCTOR;
    "if", IF;
    "in", IN;
    "include", INCLUDE;
    "inherit", INHERIT;
    "initializer", INITIALIZER;
    "lazy", LAZY;
    "let", LET;
    "match", MATCH;
    "method", METHOD;
    "module", MODULE;
    "mutable", MUTABLE;
    "new", NEW;
    "nonrec", NONREC;
    "object", OBJECT;
    "of", OF;
    "open", OPEN;
    "or", OR;
(*  "parser", PARSER; *)
    "private", PRIVATE;
    "rec", REC;
    "sig", SIG;
    "struct", STRUCT;
    "then", THEN;
    "to", TO;
    "true", TRUE;
    "try", TRY;
    "type", TYPE;
    "val", VAL;
    "virtual", VIRTUAL;
    "when", WHEN;
    "while", WHILE;
    "with", WITH;

    "mod", INFIXOP3("mod");
    "land", INFIXOP3("land");
    "lor", INFIXOP3("lor");
    "lxor", INFIXOP3("lxor");
    "lsl", INFIXOP4("lsl");
    "lsr", INFIXOP4("lsr");
    "asr", INFIXOP4("asr");
]

let keywords l = create_hashtable 11 l

(* To store the position of the beginning of a string and comment *)
let in_comment state = state.comment_start_loc <> []
let in_string state = state.string_start_loc != Location.none

(* To translate escape sequences *)

let char_for_backslash = function
  | 'n' -> '\010'
  | 'r' -> '\013'
  | 'b' -> '\008'
  | 't' -> '\009'
  | c   -> c

let char_for_decimal_code state lexbuf i =
  let c = 100 * (Char.code(Lexing.lexeme_char lexbuf i) - 48) +
           10 * (Char.code(Lexing.lexeme_char lexbuf (i+1)) - 48) +
                (Char.code(Lexing.lexeme_char lexbuf (i+2)) - 48) in
  if (c < 0 || c > 255) then
    if in_comment state
    then return 'x'
    else fail (Illegal_escape (Lexing.lexeme lexbuf))
               (Location.curr lexbuf)
  else return (Char.chr c)

let char_for_hexadecimal_code lexbuf i =
  let d1 = Char.code (Lexing.lexeme_char lexbuf i) in
  let val1 = if d1 >= 97 then d1 - 87
             else if d1 >= 65 then d1 - 55
             else d1 - 48
  in
  let d2 = Char.code (Lexing.lexeme_char lexbuf (i+1)) in
  let val2 = if d2 >= 97 then d2 - 87
             else if d2 >= 65 then d2 - 55
             else d2 - 48
  in
  Char.chr (val1 * 16 + val2)

(* To convert integer literals, allowing max_int + 1 (PR#4210) *)

let cvt_int_literal s =
  - int_of_string ("-" ^ s)
let cvt_int32_literal s =
  Int32.neg (Int32.of_string ("-" ^ String.sub s 0 (String.length s - 1)))
let cvt_int64_literal s =
  Int64.neg (Int64.of_string ("-" ^ String.sub s 0 (String.length s - 1)))
let cvt_nativeint_literal s =
  Nativeint.neg (Nativeint.of_string ("-" ^ String.sub s 0
                                                       (String.length s - 1)))

let keyword_or state s default =
  try Hashtbl.find state.keywords s
  with Not_found -> try Hashtbl.find keyword_table s
  with Not_found -> default

(* Remove underscores from float literals *)

let remove_underscores s =
  let l = String.length s in
  let rec remove src dst =
    if src >= l then
      if dst >= l then s else String.sub s 0 dst
    else
      match s.[src] with
        '_' -> remove (src + 1) dst
      |  c  -> s.[dst] <- c; remove (src + 1) (dst + 1)
  in remove 0 0

(* recover the name from a LABEL or OPTLABEL token *)

let get_label_name lexbuf =
  let s = Lexing.lexeme lexbuf in
  let name = String.sub s 1 (String.length s - 2) in
  if Hashtbl.mem keyword_table name then
    fail (Keyword_as_label name) (Location.curr lexbuf)
  else
    return name
;;

(* Update the current location with file name and line number. *)

let update_loc lexbuf file line absolute chars =
  let pos = lexbuf.lex_curr_p in
  let new_file = match file with
                 | None -> pos.pos_fname
                 | Some s -> s
  in
  lexbuf.lex_curr_p <- { pos with
    pos_fname = new_file;
    pos_lnum = if absolute then line else pos.pos_lnum + line;
    pos_bol = pos.pos_cnum - chars;
  }
;;

(* Warn about Latin-1 characters used in idents *)

let warn_latin1 lexbuf =
  Location.prerr_warning (Location.curr lexbuf)
    (Warnings.Deprecated "ISO-Latin1 characters in identifiers")
;;

(* Error report *)

open Format

let report_error ppf = function
  | Illegal_character c ->
      fprintf ppf "Illegal character (%s)" (Char.escaped c)
  | Illegal_escape s ->
      fprintf ppf "Illegal backslash escape in string or character (%s)" s
  | Unterminated_comment _ ->
      fprintf ppf "Comment not terminated"
  | Unterminated_string ->
      fprintf ppf "String literal not terminated"
  | Unterminated_string_in_comment (_, loc) ->
      fprintf ppf "This comment contains an unterminated string literal@.\
                   %aString literal begins here"
              Location.print_error loc
  | Keyword_as_label kwd ->
      fprintf ppf "`%s' is a keyword, it cannot be used as label name" kwd
  | Literal_overflow ty ->
      fprintf ppf "Integer literal exceeds the range of representable \
                   integers of type %s" ty

let () =
  Location.register_error_of_exn
    (function
      | Error (err, loc) ->
        Some (Location.error_of_printer loc report_error err)
      | _ ->
        None
    )


# 264 "src/ocaml/lexer_raw.ml"
let __ocaml_lex_refill : (Lexing.lexbuf -> 'a) -> (Lexing.lexbuf -> 'a) =
# 305 "src/ocaml/preprocess/lexer_raw.mll"
       (fun k lexbuf -> Refill (fun () -> k lexbuf))
# 268 "src/ocaml/lexer_raw.ml"
let __ocaml_lex_tables = {
  Lexing.lex_base = 
   "\000\000\162\255\163\255\094\000\153\000\224\000\003\001\038\001\
    \073\001\188\255\108\001\196\255\141\000\145\001\180\001\069\000\
    \084\000\215\001\213\255\215\255\218\255\250\001\089\002\124\002\
    \087\000\012\001\154\002\233\255\231\002\008\003\081\003\165\003\
    \117\004\069\005\036\006\128\006\080\007\047\008\133\000\135\000\
    \254\255\001\000\005\000\255\255\006\000\007\000\210\255\250\255\
    \252\255\187\255\189\255\084\008\119\008\149\008\101\009\247\255\
    \246\255\068\010\098\010\050\011\244\255\243\255\002\012\225\012\
    \192\013\234\255\235\255\236\255\120\000\011\002\085\000\004\001\
    \124\002\240\003\192\004\111\000\021\002\147\002\194\255\232\255\
    \097\005\209\013\096\000\099\000\011\000\231\255\230\255\226\255\
    \206\004\179\002\104\000\229\255\189\002\109\000\228\255\244\006\
    \110\000\227\255\109\000\224\255\234\013\223\255\045\014\194\007\
    \061\006\222\255\012\000\016\000\034\001\115\001\017\000\222\255\
    \018\000\080\014\117\014\152\014\187\014\206\255\207\255\208\255\
    \204\255\222\014\114\000\089\000\197\255\198\255\199\255\097\000\
    \182\255\184\255\191\255\001\015\036\015\071\015\106\015\141\015\
    \210\005\243\255\244\255\170\000\245\255\104\001\177\008\253\255\
    \121\000\122\000\255\255\254\255\252\255\205\008\159\015\127\000\
    \142\000\172\000\251\255\250\255\249\255\164\007\241\002\145\000\
    \248\255\013\004\153\000\247\255\176\009\155\000\246\255\194\000\
    \129\001\247\255\248\255\249\255\199\000\232\015\255\255\250\255\
    \173\010\229\004\253\255\003\001\014\001\024\001\240\004\252\255\
    \125\011\251\255\059\001\095\001\252\255\030\012\254\255\255\255\
    \060\001\069\001\253\255\059\012\162\000\169\000\198\000\200\000\
    \189\000\254\000\223\000\013\000\255\255";
  Lexing.lex_backtrk = 
   "\255\255\255\255\255\255\088\000\087\000\084\000\083\000\076\000\
    \074\000\255\255\062\000\255\255\055\000\054\000\052\000\050\000\
    \046\000\079\000\255\255\255\255\255\255\035\000\034\000\041\000\
    \039\000\038\000\060\000\255\255\017\000\017\000\016\000\015\000\
    \014\000\014\000\010\000\007\000\006\000\065\000\044\000\002\000\
    \255\255\093\000\093\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\004\000\082\000\255\255\255\255\255\255\
    \255\255\013\000\255\255\255\255\255\255\255\255\014\000\014\000\
    \090\000\255\255\255\255\255\255\255\255\018\000\255\255\255\255\
    \255\255\017\000\017\000\017\000\018\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \029\000\029\000\029\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\030\000\255\255\086\000\255\255\089\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\036\000\085\000\080\000\043\000\255\255\255\255\255\255\
    \255\255\053\000\070\000\069\000\255\255\255\255\255\255\072\000\
    \255\255\255\255\255\255\063\000\081\000\075\000\078\000\077\000\
    \255\255\255\255\255\255\012\000\255\255\012\000\012\000\255\255\
    \012\000\012\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\008\000\008\000\255\255\255\255\
    \005\000\005\000\255\255\001\000\005\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\003\000\255\255\255\255\
    \003\000\255\255\255\255\255\255\002\000\255\255\255\255\001\000\
    \255\255\255\255\255\255\255\255\255\255";
  Lexing.lex_default = 
   "\001\000\000\000\000\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\255\255\000\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\000\000\000\000\000\000\255\255\255\255\255\255\
    \255\255\082\000\255\255\000\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \000\000\255\255\255\255\000\000\255\255\255\255\000\000\000\000\
    \000\000\000\000\000\000\255\255\255\255\255\255\255\255\000\000\
    \000\000\255\255\255\255\255\255\000\000\000\000\255\255\255\255\
    \255\255\000\000\000\000\000\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\000\000\000\000\
    \255\255\087\000\255\255\255\255\255\255\000\000\000\000\000\000\
    \255\255\255\255\255\255\000\000\255\255\255\255\000\000\255\255\
    \255\255\000\000\255\255\000\000\255\255\000\000\255\255\107\000\
    \255\255\000\000\255\255\107\000\108\000\107\000\110\000\000\000\
    \255\255\255\255\255\255\255\255\255\255\000\000\000\000\000\000\
    \000\000\255\255\255\255\255\255\000\000\000\000\000\000\255\255\
    \000\000\000\000\000\000\255\255\255\255\255\255\255\255\255\255\
    \137\000\000\000\000\000\255\255\000\000\151\000\255\255\000\000\
    \255\255\255\255\000\000\000\000\000\000\255\255\255\255\255\255\
    \255\255\255\255\000\000\000\000\000\000\255\255\255\255\255\255\
    \000\000\255\255\255\255\000\000\255\255\255\255\000\000\255\255\
    \169\000\000\000\000\000\000\000\255\255\175\000\000\000\000\000\
    \255\255\255\255\000\000\255\255\255\255\255\255\255\255\000\000\
    \255\255\000\000\255\255\188\000\000\000\255\255\000\000\000\000\
    \255\255\255\255\000\000\255\255\255\255\255\255\198\000\201\000\
    \255\255\201\000\255\255\255\255\000\000";
  Lexing.lex_trans = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\039\000\040\000\040\000\039\000\041\000\045\000\043\000\
    \043\000\040\000\044\000\044\000\045\000\083\000\105\000\204\000\
    \084\000\106\000\105\000\111\000\111\000\106\000\112\000\112\000\
    \039\000\008\000\027\000\022\000\006\000\004\000\021\000\025\000\
    \024\000\019\000\023\000\007\000\018\000\017\000\038\000\003\000\
    \029\000\028\000\028\000\028\000\028\000\028\000\028\000\028\000\
    \028\000\028\000\016\000\015\000\014\000\013\000\037\000\034\000\
    \005\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\012\000\042\000\011\000\005\000\036\000\
    \020\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\033\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\026\000\010\000\009\000\035\000\003\000\
    \120\000\098\000\003\000\003\000\003\000\075\000\075\000\086\000\
    \003\000\003\000\085\000\003\000\003\000\003\000\119\000\091\000\
    \039\000\118\000\117\000\039\000\094\000\097\000\099\000\129\000\
    \003\000\127\000\003\000\003\000\003\000\003\000\003\000\075\000\
    \075\000\128\000\147\000\077\000\146\000\077\000\156\000\039\000\
    \076\000\076\000\076\000\076\000\076\000\076\000\076\000\076\000\
    \076\000\076\000\122\000\046\000\138\000\155\000\152\000\167\000\
    \160\000\153\000\003\000\066\000\003\000\003\000\003\000\003\000\
    \163\000\048\000\166\000\003\000\003\000\197\000\003\000\003\000\
    \003\000\125\000\198\000\124\000\138\000\123\000\075\000\167\000\
    \199\000\171\000\200\000\003\000\186\000\003\000\003\000\003\000\
    \003\000\003\000\003\000\067\000\003\000\065\000\202\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\003\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \002\000\005\000\203\000\047\000\005\000\005\000\005\000\000\000\
    \200\000\126\000\005\000\005\000\179\000\005\000\005\000\005\000\
    \255\255\255\255\000\000\000\000\000\000\003\000\083\000\003\000\
    \179\000\084\000\005\000\181\000\005\000\005\000\005\000\005\000\
    \005\000\000\000\179\000\179\000\006\000\181\000\000\000\006\000\
    \006\000\006\000\000\000\000\000\105\000\006\000\006\000\106\000\
    \006\000\006\000\006\000\255\255\074\000\074\000\074\000\074\000\
    \074\000\074\000\074\000\074\000\000\000\006\000\005\000\006\000\
    \006\000\006\000\006\000\006\000\110\000\171\000\191\000\114\000\
    \186\000\193\000\114\000\114\000\114\000\000\000\000\000\191\000\
    \114\000\114\000\193\000\114\000\135\000\114\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\005\000\000\000\005\000\000\000\
    \114\000\006\000\114\000\134\000\114\000\114\000\114\000\000\000\
    \081\000\191\000\132\000\000\000\192\000\132\000\132\000\132\000\
    \000\000\000\000\152\000\132\000\132\000\153\000\132\000\132\000\
    \132\000\000\000\000\000\000\000\109\000\105\000\000\000\006\000\
    \106\000\006\000\000\000\132\000\114\000\132\000\133\000\132\000\
    \132\000\132\000\000\000\171\000\000\000\006\000\172\000\154\000\
    \006\000\006\000\006\000\109\000\000\000\108\000\006\000\006\000\
    \000\000\006\000\006\000\006\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\114\000\174\000\114\000\000\000\006\000\132\000\
    \006\000\006\000\006\000\006\000\006\000\000\000\000\000\000\000\
    \000\000\000\000\006\000\000\000\000\000\006\000\006\000\006\000\
    \000\000\000\000\000\000\006\000\006\000\000\000\006\000\006\000\
    \006\000\000\000\000\000\000\000\150\000\132\000\255\255\132\000\
    \255\255\130\000\006\000\006\000\000\000\006\000\006\000\006\000\
    \006\000\006\000\000\000\000\000\000\000\006\000\000\000\000\000\
    \006\000\006\000\006\000\189\000\000\000\173\000\006\000\006\000\
    \000\000\121\000\006\000\006\000\000\000\000\000\000\000\000\000\
    \131\000\000\000\006\000\000\000\000\000\000\000\006\000\006\000\
    \006\000\006\000\006\000\006\000\006\000\000\000\000\000\000\000\
    \114\000\000\000\000\000\114\000\114\000\114\000\255\255\000\000\
    \000\000\114\000\114\000\000\000\114\000\115\000\114\000\000\000\
    \000\000\000\000\000\000\000\000\255\255\006\000\000\000\006\000\
    \000\000\114\000\006\000\114\000\114\000\116\000\114\000\114\000\
    \000\000\000\000\000\000\006\000\000\000\000\000\006\000\006\000\
    \113\000\000\000\255\255\000\000\006\000\006\000\000\000\006\000\
    \006\000\006\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \006\000\000\000\006\000\000\000\006\000\114\000\006\000\006\000\
    \006\000\006\000\006\000\069\000\069\000\069\000\069\000\069\000\
    \069\000\069\000\069\000\069\000\069\000\076\000\076\000\076\000\
    \076\000\076\000\076\000\076\000\076\000\076\000\076\000\000\000\
    \068\000\000\000\000\000\114\000\000\000\114\000\000\000\000\000\
    \006\000\000\000\000\000\000\000\000\000\000\000\000\000\190\000\
    \000\000\000\000\104\000\000\000\000\000\000\000\000\000\000\000\
    \255\255\000\000\069\000\000\000\000\000\000\000\000\000\000\000\
    \068\000\000\000\000\000\255\255\076\000\000\000\006\000\000\000\
    \006\000\104\000\102\000\000\000\102\000\102\000\102\000\102\000\
    \000\000\170\000\000\000\102\000\102\000\000\000\102\000\102\000\
    \102\000\103\000\103\000\103\000\103\000\103\000\103\000\103\000\
    \103\000\103\000\103\000\102\000\000\000\102\000\102\000\102\000\
    \102\000\102\000\000\000\000\000\000\000\003\000\000\000\000\000\
    \003\000\003\000\003\000\000\000\000\000\101\000\100\000\003\000\
    \000\000\003\000\003\000\003\000\073\000\073\000\073\000\073\000\
    \073\000\073\000\073\000\073\000\073\000\073\000\003\000\102\000\
    \003\000\003\000\003\000\003\000\003\000\073\000\073\000\073\000\
    \073\000\073\000\073\000\076\000\076\000\076\000\076\000\076\000\
    \076\000\076\000\076\000\076\000\076\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\102\000\078\000\102\000\
    \000\000\000\000\003\000\000\000\000\000\073\000\073\000\073\000\
    \073\000\073\000\073\000\092\000\092\000\092\000\092\000\092\000\
    \092\000\092\000\092\000\092\000\092\000\093\000\093\000\093\000\
    \093\000\093\000\093\000\093\000\093\000\093\000\093\000\000\000\
    \003\000\080\000\003\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\069\000\079\000\028\000\
    \028\000\028\000\028\000\028\000\028\000\028\000\028\000\028\000\
    \028\000\161\000\161\000\161\000\161\000\161\000\161\000\161\000\
    \161\000\161\000\161\000\000\000\068\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\066\000\000\000\000\000\069\000\000\000\
    \028\000\028\000\028\000\028\000\028\000\028\000\028\000\028\000\
    \028\000\028\000\000\000\000\000\000\000\000\000\028\000\000\000\
    \000\000\000\000\070\000\000\000\068\000\068\000\000\000\000\000\
    \000\000\000\000\000\000\067\000\066\000\065\000\000\000\071\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \072\000\000\000\000\000\000\000\000\000\000\000\000\000\028\000\
    \000\000\000\000\070\000\000\000\000\000\068\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\067\000\000\000\065\000\071\000\
    \030\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \072\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\000\000\000\000\000\000\000\000\
    \030\000\000\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\031\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \000\000\000\000\000\000\000\000\031\000\000\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \073\000\073\000\073\000\073\000\073\000\073\000\073\000\073\000\
    \073\000\073\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\073\000\073\000\073\000\073\000\073\000\073\000\000\000\
    \000\000\000\000\000\000\000\000\066\000\162\000\162\000\162\000\
    \162\000\162\000\162\000\162\000\162\000\162\000\162\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\073\000\
    \000\000\073\000\073\000\073\000\073\000\073\000\073\000\000\000\
    \000\000\000\000\000\000\000\000\067\000\000\000\065\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\000\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\032\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \000\000\000\000\000\000\000\000\032\000\000\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \074\000\074\000\074\000\074\000\074\000\074\000\074\000\074\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\095\000\095\000\
    \095\000\095\000\095\000\095\000\095\000\095\000\095\000\095\000\
    \000\000\000\000\000\000\000\000\066\000\000\000\000\000\095\000\
    \095\000\095\000\095\000\095\000\095\000\182\000\182\000\182\000\
    \182\000\182\000\182\000\182\000\182\000\182\000\182\000\074\000\
    \183\000\183\000\183\000\183\000\183\000\183\000\183\000\183\000\
    \183\000\183\000\000\000\000\000\067\000\000\000\065\000\095\000\
    \095\000\095\000\095\000\095\000\095\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\000\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\032\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \000\000\000\000\000\000\000\000\032\000\000\000\032\000\032\000\
    \032\000\032\000\062\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \080\000\000\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\138\000\079\000\000\000\139\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\143\000\000\000\000\000\000\000\
    \000\000\141\000\145\000\000\000\144\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\000\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\000\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\052\000\104\000\000\000\
    \052\000\052\000\052\000\000\000\000\000\142\000\052\000\052\000\
    \000\000\052\000\052\000\052\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\104\000\052\000\000\000\
    \052\000\052\000\052\000\057\000\052\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\103\000\103\000\103\000\
    \103\000\103\000\103\000\103\000\103\000\103\000\103\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\052\000\059\000\000\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\000\000\
    \052\000\052\000\052\000\000\000\052\000\052\000\052\000\000\000\
    \000\000\000\000\052\000\052\000\000\000\052\000\052\000\052\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\052\000\000\000\052\000\052\000\052\000\052\000\
    \052\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\140\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\052\000\054\000\
    \000\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\000\000\052\000\000\000\052\000\000\000\
    \000\000\000\000\000\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\000\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\096\000\096\000\096\000\096\000\
    \096\000\096\000\096\000\096\000\096\000\096\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\096\000\096\000\096\000\
    \096\000\096\000\096\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\096\000\096\000\096\000\
    \096\000\096\000\096\000\000\000\000\000\000\000\000\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\032\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\000\000\000\000\000\000\000\000\032\000\
    \000\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\109\000\105\000\000\000\000\000\106\000\
    \000\000\000\000\000\000\000\000\164\000\164\000\164\000\164\000\
    \164\000\164\000\164\000\164\000\164\000\164\000\000\000\000\000\
    \000\000\000\000\109\000\000\000\108\000\164\000\164\000\164\000\
    \164\000\164\000\164\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\103\000\103\000\103\000\103\000\103\000\103\000\
    \103\000\103\000\103\000\103\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\164\000\164\000\164\000\
    \164\000\164\000\164\000\000\000\000\000\000\000\000\000\000\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\000\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\000\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \006\000\000\000\000\000\006\000\006\000\006\000\000\000\000\000\
    \000\000\006\000\006\000\000\000\006\000\051\000\006\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\006\000\000\000\006\000\006\000\006\000\006\000\006\000\
    \000\000\000\000\000\000\000\000\000\000\006\000\000\000\000\000\
    \006\000\006\000\006\000\000\000\000\000\000\000\006\000\006\000\
    \000\000\006\000\006\000\006\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\050\000\006\000\006\000\000\000\
    \006\000\006\000\006\000\006\000\006\000\000\000\000\000\000\000\
    \052\000\000\000\000\000\052\000\052\000\052\000\000\000\000\000\
    \000\000\052\000\052\000\000\000\052\000\052\000\052\000\000\000\
    \000\000\000\000\000\000\006\000\049\000\006\000\000\000\000\000\
    \000\000\052\000\006\000\052\000\052\000\052\000\052\000\052\000\
    \000\000\000\000\000\000\000\000\053\000\000\000\000\000\000\000\
    \000\000\000\000\255\255\000\000\000\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\056\000\
    \006\000\000\000\006\000\000\000\000\000\052\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \000\000\000\000\000\000\052\000\053\000\052\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \149\000\000\000\149\000\149\000\149\000\149\000\149\000\149\000\
    \149\000\149\000\149\000\149\000\149\000\149\000\149\000\149\000\
    \149\000\149\000\149\000\149\000\149\000\149\000\149\000\149\000\
    \149\000\149\000\149\000\149\000\149\000\148\000\149\000\149\000\
    \149\000\149\000\149\000\149\000\149\000\149\000\149\000\149\000\
    \149\000\149\000\149\000\149\000\149\000\149\000\149\000\149\000\
    \149\000\149\000\149\000\149\000\149\000\149\000\149\000\149\000\
    \000\000\148\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\000\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\054\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\055\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \000\000\000\000\000\000\000\000\054\000\000\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \165\000\165\000\165\000\165\000\165\000\165\000\165\000\165\000\
    \165\000\165\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\165\000\165\000\165\000\165\000\165\000\165\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\165\000\165\000\165\000\165\000\165\000\165\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\000\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\000\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\052\000\000\000\000\000\
    \052\000\052\000\052\000\000\000\000\000\000\000\052\000\052\000\
    \000\000\052\000\052\000\052\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\052\000\000\000\
    \052\000\052\000\052\000\052\000\052\000\000\000\000\000\000\000\
    \000\000\058\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\061\000\000\000\000\000\000\000\
    \000\000\000\000\052\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\000\000\000\000\000\000\
    \052\000\058\000\052\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\184\000\184\000\184\000\
    \184\000\184\000\184\000\184\000\184\000\184\000\184\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\184\000\184\000\
    \184\000\184\000\184\000\184\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\184\000\184\000\
    \184\000\184\000\184\000\184\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\000\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\059\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\060\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\000\000\000\000\000\000\
    \000\000\059\000\000\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\185\000\185\000\185\000\
    \185\000\185\000\185\000\185\000\185\000\185\000\185\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\185\000\185\000\
    \185\000\185\000\185\000\185\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\185\000\185\000\
    \185\000\185\000\185\000\185\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\000\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\032\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\000\000\000\000\000\000\
    \000\000\032\000\000\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\063\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\195\000\000\000\195\000\
    \195\000\195\000\195\000\195\000\195\000\195\000\195\000\195\000\
    \195\000\195\000\195\000\195\000\195\000\195\000\195\000\195\000\
    \195\000\195\000\195\000\195\000\195\000\195\000\195\000\195\000\
    \195\000\000\000\195\000\194\000\195\000\195\000\195\000\195\000\
    \195\000\195\000\195\000\195\000\195\000\195\000\195\000\195\000\
    \195\000\195\000\195\000\195\000\195\000\195\000\195\000\195\000\
    \195\000\195\000\195\000\195\000\195\000\195\000\000\000\000\000\
    \194\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\000\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\000\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\064\000\000\000\000\000\064\000\000\000\064\000\
    \032\000\000\000\000\000\064\000\064\000\000\000\064\000\064\000\
    \064\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\064\000\000\000\064\000\064\000\064\000\
    \064\000\064\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\000\000\000\000\000\000\064\000\
    \032\000\000\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\000\000\064\000\000\000\064\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \000\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \000\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\064\000\000\000\000\000\064\000\064\000\064\000\000\000\
    \000\000\000\000\064\000\064\000\000\000\064\000\064\000\064\000\
    \000\000\090\000\000\000\090\000\000\000\000\000\000\000\000\000\
    \090\000\000\000\064\000\000\000\064\000\064\000\064\000\064\000\
    \064\000\089\000\089\000\089\000\089\000\089\000\089\000\089\000\
    \089\000\089\000\089\000\100\000\000\000\000\000\100\000\100\000\
    \100\000\000\000\000\000\000\000\100\000\100\000\000\000\100\000\
    \100\000\100\000\000\000\000\000\000\000\000\000\064\000\000\000\
    \000\000\000\000\000\000\000\000\100\000\000\000\100\000\100\000\
    \100\000\100\000\100\000\000\000\000\000\090\000\000\000\000\000\
    \000\000\000\000\000\000\090\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\064\000\000\000\064\000\090\000\
    \000\000\000\000\000\000\090\000\000\000\090\000\000\000\000\000\
    \100\000\088\000\000\000\000\000\000\000\000\000\102\000\000\000\
    \102\000\102\000\102\000\102\000\000\000\000\000\000\000\102\000\
    \102\000\000\000\102\000\102\000\102\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\100\000\102\000\
    \100\000\102\000\102\000\102\000\102\000\102\000\000\000\000\000\
    \000\000\006\000\000\000\000\000\006\000\006\000\006\000\000\000\
    \000\000\000\000\006\000\006\000\000\000\006\000\006\000\006\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\006\000\102\000\006\000\006\000\006\000\006\000\
    \006\000\000\000\000\000\000\000\000\000\000\000\114\000\000\000\
    \000\000\114\000\114\000\114\000\000\000\000\000\000\000\114\000\
    \114\000\000\000\114\000\114\000\114\000\000\000\000\000\000\000\
    \000\000\102\000\000\000\102\000\000\000\000\000\006\000\114\000\
    \000\000\114\000\114\000\114\000\114\000\114\000\000\000\000\000\
    \000\000\114\000\000\000\000\000\114\000\114\000\114\000\000\000\
    \000\000\000\000\114\000\114\000\000\000\114\000\114\000\114\000\
    \000\000\000\000\000\000\000\000\006\000\000\000\006\000\000\000\
    \000\000\255\255\114\000\114\000\114\000\114\000\114\000\114\000\
    \114\000\000\000\000\000\000\000\114\000\000\000\000\000\114\000\
    \114\000\114\000\000\000\000\000\000\000\114\000\114\000\000\000\
    \114\000\114\000\114\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\114\000\000\000\114\000\000\000\114\000\114\000\114\000\
    \114\000\114\000\114\000\114\000\000\000\000\000\000\000\006\000\
    \000\000\000\000\006\000\006\000\006\000\000\000\000\000\000\000\
    \006\000\006\000\000\000\006\000\006\000\006\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\114\000\000\000\114\000\000\000\
    \006\000\114\000\006\000\006\000\006\000\006\000\006\000\000\000\
    \000\000\000\000\006\000\000\000\000\000\006\000\006\000\006\000\
    \000\000\000\000\000\000\006\000\006\000\000\000\006\000\006\000\
    \006\000\000\000\000\000\000\000\000\000\000\000\000\000\114\000\
    \000\000\114\000\000\000\006\000\006\000\006\000\006\000\006\000\
    \006\000\006\000\000\000\000\000\000\000\132\000\000\000\000\000\
    \132\000\132\000\132\000\000\000\000\000\000\000\132\000\132\000\
    \000\000\132\000\132\000\132\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\006\000\000\000\006\000\000\000\132\000\006\000\
    \132\000\132\000\132\000\132\000\132\000\000\000\000\000\000\000\
    \132\000\000\000\000\000\132\000\132\000\132\000\000\000\000\000\
    \000\000\132\000\132\000\000\000\132\000\132\000\132\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\006\000\000\000\006\000\
    \000\000\132\000\132\000\132\000\132\000\132\000\132\000\132\000\
    \000\000\000\000\000\000\114\000\000\000\000\000\114\000\114\000\
    \114\000\000\000\000\000\000\000\114\000\114\000\000\000\114\000\
    \114\000\114\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \132\000\000\000\132\000\000\000\114\000\132\000\114\000\114\000\
    \114\000\114\000\114\000\000\000\000\000\000\000\114\000\000\000\
    \000\000\114\000\114\000\114\000\000\000\000\000\000\000\114\000\
    \114\000\000\000\114\000\114\000\114\000\000\000\000\000\159\000\
    \000\000\159\000\000\000\132\000\000\000\132\000\159\000\114\000\
    \114\000\114\000\114\000\114\000\114\000\114\000\000\000\158\000\
    \158\000\158\000\158\000\158\000\158\000\158\000\158\000\158\000\
    \158\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\114\000\000\000\
    \114\000\000\000\000\000\114\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\179\000\000\000\000\000\180\000\000\000\000\000\
    \000\000\000\000\000\000\159\000\000\000\000\000\000\000\000\000\
    \000\000\159\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \178\000\114\000\178\000\114\000\000\000\159\000\000\000\178\000\
    \000\000\159\000\000\000\159\000\000\000\000\000\000\000\157\000\
    \177\000\177\000\177\000\177\000\177\000\177\000\177\000\177\000\
    \177\000\177\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\178\000\000\000\000\000\000\000\
    \000\000\000\000\178\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\178\000\000\000\
    \000\000\000\000\178\000\000\000\178\000\000\000\000\000\000\000\
    \176\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \255\255";
  Lexing.lex_check = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\000\000\000\000\041\000\000\000\000\000\041\000\042\000\
    \044\000\045\000\042\000\044\000\045\000\084\000\106\000\203\000\
    \084\000\106\000\107\000\110\000\112\000\107\000\110\000\112\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\003\000\
    \015\000\024\000\003\000\003\000\003\000\070\000\070\000\082\000\
    \003\000\003\000\083\000\003\000\003\000\003\000\016\000\090\000\
    \039\000\016\000\016\000\039\000\093\000\096\000\098\000\122\000\
    \003\000\123\000\003\000\003\000\003\000\003\000\003\000\075\000\
    \075\000\127\000\144\000\068\000\145\000\068\000\151\000\039\000\
    \068\000\068\000\068\000\068\000\068\000\068\000\068\000\068\000\
    \068\000\068\000\012\000\038\000\139\000\152\000\153\000\139\000\
    \159\000\153\000\004\000\075\000\003\000\004\000\004\000\004\000\
    \162\000\038\000\165\000\004\000\004\000\196\000\004\000\004\000\
    \004\000\012\000\197\000\012\000\167\000\012\000\075\000\167\000\
    \198\000\172\000\199\000\004\000\172\000\004\000\004\000\004\000\
    \004\000\004\000\003\000\075\000\003\000\075\000\200\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\004\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\005\000\202\000\038\000\005\000\005\000\005\000\255\255\
    \201\000\012\000\005\000\005\000\179\000\005\000\005\000\005\000\
    \107\000\110\000\255\255\255\255\255\255\004\000\025\000\004\000\
    \180\000\025\000\005\000\180\000\005\000\005\000\005\000\005\000\
    \005\000\255\255\181\000\179\000\006\000\181\000\255\255\006\000\
    \006\000\006\000\255\255\255\255\108\000\006\000\006\000\108\000\
    \006\000\006\000\006\000\025\000\071\000\071\000\071\000\071\000\
    \071\000\071\000\071\000\071\000\255\255\006\000\005\000\006\000\
    \006\000\006\000\006\000\006\000\108\000\186\000\192\000\007\000\
    \186\000\192\000\007\000\007\000\007\000\255\255\255\255\193\000\
    \007\000\007\000\193\000\007\000\007\000\007\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\005\000\255\255\005\000\255\255\
    \007\000\006\000\007\000\007\000\007\000\007\000\007\000\255\255\
    \025\000\187\000\008\000\255\255\187\000\008\000\008\000\008\000\
    \255\255\255\255\141\000\008\000\008\000\141\000\008\000\008\000\
    \008\000\255\255\255\255\255\255\109\000\109\000\255\255\006\000\
    \109\000\006\000\255\255\008\000\007\000\008\000\008\000\008\000\
    \008\000\008\000\255\255\168\000\255\255\010\000\168\000\141\000\
    \010\000\010\000\010\000\109\000\255\255\109\000\010\000\010\000\
    \255\255\010\000\010\000\010\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\007\000\168\000\007\000\255\255\010\000\008\000\
    \010\000\010\000\010\000\010\000\010\000\255\255\255\255\255\255\
    \255\255\255\255\013\000\255\255\255\255\013\000\013\000\013\000\
    \255\255\255\255\255\255\013\000\013\000\255\255\013\000\013\000\
    \013\000\255\255\255\255\255\255\141\000\008\000\198\000\008\000\
    \199\000\010\000\010\000\013\000\255\255\013\000\013\000\013\000\
    \013\000\013\000\255\255\255\255\255\255\014\000\255\255\255\255\
    \014\000\014\000\014\000\187\000\255\255\168\000\014\000\014\000\
    \255\255\014\000\014\000\014\000\255\255\255\255\255\255\255\255\
    \010\000\255\255\010\000\255\255\255\255\255\255\014\000\013\000\
    \014\000\014\000\014\000\014\000\014\000\255\255\255\255\255\255\
    \017\000\255\255\255\255\017\000\017\000\017\000\201\000\255\255\
    \255\255\017\000\017\000\255\255\017\000\017\000\017\000\255\255\
    \255\255\255\255\255\255\255\255\025\000\013\000\255\255\013\000\
    \255\255\017\000\014\000\017\000\017\000\017\000\017\000\017\000\
    \255\255\255\255\255\255\021\000\255\255\255\255\021\000\021\000\
    \021\000\255\255\108\000\255\255\021\000\021\000\255\255\021\000\
    \021\000\021\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \014\000\255\255\014\000\255\255\021\000\017\000\021\000\021\000\
    \021\000\021\000\021\000\069\000\069\000\069\000\069\000\069\000\
    \069\000\069\000\069\000\069\000\069\000\076\000\076\000\076\000\
    \076\000\076\000\076\000\076\000\076\000\076\000\076\000\255\255\
    \069\000\255\255\255\255\017\000\255\255\017\000\255\255\255\255\
    \021\000\255\255\255\255\255\255\255\255\255\255\255\255\187\000\
    \255\255\255\255\022\000\255\255\255\255\255\255\255\255\255\255\
    \141\000\255\255\069\000\255\255\255\255\255\255\255\255\255\255\
    \069\000\255\255\255\255\109\000\076\000\255\255\021\000\255\255\
    \021\000\022\000\022\000\255\255\022\000\022\000\022\000\022\000\
    \255\255\168\000\255\255\022\000\022\000\255\255\022\000\022\000\
    \022\000\022\000\022\000\022\000\022\000\022\000\022\000\022\000\
    \022\000\022\000\022\000\022\000\255\255\022\000\022\000\022\000\
    \022\000\022\000\255\255\255\255\255\255\023\000\255\255\255\255\
    \023\000\023\000\023\000\255\255\255\255\023\000\023\000\023\000\
    \255\255\023\000\023\000\023\000\072\000\072\000\072\000\072\000\
    \072\000\072\000\072\000\072\000\072\000\072\000\023\000\022\000\
    \023\000\023\000\023\000\023\000\023\000\072\000\072\000\072\000\
    \072\000\072\000\072\000\077\000\077\000\077\000\077\000\077\000\
    \077\000\077\000\077\000\077\000\077\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\022\000\026\000\022\000\
    \255\255\255\255\023\000\255\255\255\255\072\000\072\000\072\000\
    \072\000\072\000\072\000\089\000\089\000\089\000\089\000\089\000\
    \089\000\089\000\089\000\089\000\089\000\092\000\092\000\092\000\
    \092\000\092\000\092\000\092\000\092\000\092\000\092\000\255\255\
    \023\000\026\000\023\000\026\000\026\000\026\000\026\000\026\000\
    \026\000\026\000\026\000\026\000\026\000\026\000\026\000\026\000\
    \026\000\026\000\026\000\026\000\026\000\026\000\026\000\026\000\
    \026\000\026\000\026\000\026\000\026\000\028\000\026\000\028\000\
    \028\000\028\000\028\000\028\000\028\000\028\000\028\000\028\000\
    \028\000\158\000\158\000\158\000\158\000\158\000\158\000\158\000\
    \158\000\158\000\158\000\255\255\028\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\028\000\255\255\255\255\029\000\255\255\
    \029\000\029\000\029\000\029\000\029\000\029\000\029\000\029\000\
    \029\000\029\000\255\255\255\255\255\255\255\255\028\000\255\255\
    \255\255\255\255\029\000\255\255\028\000\029\000\255\255\255\255\
    \255\255\255\255\255\255\028\000\029\000\028\000\255\255\029\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \029\000\255\255\255\255\255\255\255\255\255\255\255\255\029\000\
    \255\255\255\255\029\000\255\255\255\255\029\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\029\000\255\255\029\000\029\000\
    \030\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \029\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\255\255\255\255\255\255\255\255\
    \030\000\255\255\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\030\000\030\000\030\000\030\000\
    \030\000\030\000\030\000\030\000\031\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \255\255\255\255\255\255\255\255\031\000\255\255\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \073\000\073\000\073\000\073\000\073\000\073\000\073\000\073\000\
    \073\000\073\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\073\000\073\000\073\000\073\000\073\000\073\000\255\255\
    \255\255\255\255\255\255\255\255\073\000\161\000\161\000\161\000\
    \161\000\161\000\161\000\161\000\161\000\161\000\161\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\073\000\
    \255\255\073\000\073\000\073\000\073\000\073\000\073\000\255\255\
    \255\255\255\255\255\255\255\255\073\000\255\255\073\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\255\255\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\032\000\031\000\031\000\031\000\
    \031\000\031\000\031\000\031\000\031\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \255\255\255\255\255\255\255\255\032\000\255\255\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \074\000\074\000\074\000\074\000\074\000\074\000\074\000\074\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\088\000\088\000\
    \088\000\088\000\088\000\088\000\088\000\088\000\088\000\088\000\
    \255\255\255\255\255\255\255\255\074\000\255\255\255\255\088\000\
    \088\000\088\000\088\000\088\000\088\000\177\000\177\000\177\000\
    \177\000\177\000\177\000\177\000\177\000\177\000\177\000\074\000\
    \182\000\182\000\182\000\182\000\182\000\182\000\182\000\182\000\
    \182\000\182\000\255\255\255\255\074\000\255\255\074\000\088\000\
    \088\000\088\000\088\000\088\000\088\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\255\255\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\033\000\032\000\032\000\032\000\
    \032\000\032\000\032\000\032\000\032\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \255\255\255\255\255\255\255\255\033\000\255\255\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \080\000\255\255\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\080\000\080\000\080\000\080\000\
    \080\000\080\000\080\000\080\000\136\000\080\000\255\255\136\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\136\000\255\255\255\255\255\255\
    \255\255\136\000\136\000\255\255\136\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\255\255\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\255\255\033\000\033\000\033\000\
    \033\000\033\000\033\000\033\000\033\000\034\000\104\000\255\255\
    \034\000\034\000\034\000\255\255\255\255\136\000\034\000\034\000\
    \255\255\034\000\034\000\034\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\104\000\034\000\255\255\
    \034\000\034\000\034\000\034\000\034\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\104\000\104\000\104\000\
    \104\000\104\000\104\000\104\000\104\000\104\000\104\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\034\000\034\000\255\255\034\000\034\000\034\000\
    \034\000\034\000\034\000\034\000\034\000\034\000\034\000\034\000\
    \034\000\034\000\034\000\034\000\034\000\034\000\034\000\034\000\
    \034\000\034\000\034\000\034\000\034\000\034\000\034\000\255\255\
    \034\000\035\000\034\000\255\255\035\000\035\000\035\000\255\255\
    \255\255\255\255\035\000\035\000\255\255\035\000\035\000\035\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\035\000\255\255\035\000\035\000\035\000\035\000\
    \035\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\136\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\035\000\035\000\
    \255\255\035\000\035\000\035\000\035\000\035\000\035\000\035\000\
    \035\000\035\000\035\000\035\000\035\000\035\000\035\000\035\000\
    \035\000\035\000\035\000\035\000\035\000\035\000\035\000\035\000\
    \035\000\035\000\035\000\255\255\035\000\255\255\035\000\255\255\
    \255\255\255\255\255\255\034\000\034\000\034\000\034\000\034\000\
    \034\000\034\000\034\000\034\000\034\000\034\000\034\000\034\000\
    \034\000\034\000\034\000\034\000\034\000\034\000\034\000\034\000\
    \034\000\034\000\034\000\255\255\034\000\034\000\034\000\034\000\
    \034\000\034\000\034\000\034\000\095\000\095\000\095\000\095\000\
    \095\000\095\000\095\000\095\000\095\000\095\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\095\000\095\000\095\000\
    \095\000\095\000\095\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\095\000\095\000\095\000\
    \095\000\095\000\095\000\255\255\255\255\255\255\255\255\035\000\
    \035\000\035\000\035\000\035\000\035\000\035\000\035\000\035\000\
    \035\000\035\000\035\000\035\000\035\000\035\000\035\000\035\000\
    \035\000\035\000\035\000\035\000\035\000\035\000\035\000\036\000\
    \035\000\035\000\035\000\035\000\035\000\035\000\035\000\035\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\255\255\255\255\255\255\255\255\036\000\
    \255\255\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\103\000\103\000\255\255\255\255\103\000\
    \255\255\255\255\255\255\255\255\157\000\157\000\157\000\157\000\
    \157\000\157\000\157\000\157\000\157\000\157\000\255\255\255\255\
    \255\255\255\255\103\000\255\255\103\000\157\000\157\000\157\000\
    \157\000\157\000\157\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\103\000\103\000\103\000\103\000\103\000\103\000\
    \103\000\103\000\103\000\103\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\157\000\157\000\157\000\
    \157\000\157\000\157\000\255\255\255\255\255\255\255\255\255\255\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\255\255\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\255\255\
    \036\000\036\000\036\000\036\000\036\000\036\000\036\000\036\000\
    \037\000\255\255\255\255\037\000\037\000\037\000\255\255\255\255\
    \255\255\037\000\037\000\255\255\037\000\037\000\037\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\037\000\255\255\037\000\037\000\037\000\037\000\037\000\
    \255\255\255\255\255\255\255\255\255\255\051\000\255\255\255\255\
    \051\000\051\000\051\000\255\255\255\255\255\255\051\000\051\000\
    \255\255\051\000\051\000\051\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\037\000\037\000\051\000\255\255\
    \051\000\051\000\051\000\051\000\051\000\255\255\255\255\255\255\
    \052\000\255\255\255\255\052\000\052\000\052\000\255\255\255\255\
    \255\255\052\000\052\000\255\255\052\000\052\000\052\000\255\255\
    \255\255\255\255\255\255\037\000\037\000\037\000\255\255\255\255\
    \255\255\052\000\051\000\052\000\052\000\052\000\052\000\052\000\
    \255\255\255\255\255\255\255\255\053\000\255\255\255\255\255\255\
    \255\255\255\255\103\000\255\255\255\255\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \051\000\255\255\051\000\255\255\255\255\052\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \255\255\255\255\255\255\052\000\053\000\052\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \142\000\255\255\142\000\142\000\142\000\142\000\142\000\142\000\
    \142\000\142\000\142\000\142\000\142\000\142\000\142\000\142\000\
    \142\000\142\000\142\000\142\000\142\000\142\000\142\000\142\000\
    \142\000\142\000\142\000\142\000\149\000\142\000\149\000\149\000\
    \149\000\149\000\149\000\149\000\149\000\149\000\149\000\149\000\
    \149\000\149\000\149\000\149\000\149\000\149\000\149\000\149\000\
    \149\000\149\000\149\000\149\000\149\000\149\000\149\000\149\000\
    \255\255\149\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\255\255\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\054\000\053\000\053\000\053\000\
    \053\000\053\000\053\000\053\000\053\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \255\255\255\255\255\255\255\255\054\000\255\255\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \164\000\164\000\164\000\164\000\164\000\164\000\164\000\164\000\
    \164\000\164\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\164\000\164\000\164\000\164\000\164\000\164\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\164\000\164\000\164\000\164\000\164\000\164\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\255\255\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\255\255\054\000\054\000\054\000\
    \054\000\054\000\054\000\054\000\054\000\057\000\255\255\255\255\
    \057\000\057\000\057\000\255\255\255\255\255\255\057\000\057\000\
    \255\255\057\000\057\000\057\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\057\000\255\255\
    \057\000\057\000\057\000\057\000\057\000\255\255\255\255\255\255\
    \255\255\058\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\255\255\255\255\255\255\
    \255\255\255\255\057\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\255\255\255\255\255\255\
    \057\000\058\000\057\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\176\000\176\000\176\000\
    \176\000\176\000\176\000\176\000\176\000\176\000\176\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\176\000\176\000\
    \176\000\176\000\176\000\176\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\176\000\176\000\
    \176\000\176\000\176\000\176\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\255\255\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\059\000\058\000\058\000\058\000\058\000\058\000\058\000\
    \058\000\058\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\255\255\255\255\255\255\
    \255\255\059\000\255\255\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\184\000\184\000\184\000\
    \184\000\184\000\184\000\184\000\184\000\184\000\184\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\184\000\184\000\
    \184\000\184\000\184\000\184\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\184\000\184\000\
    \184\000\184\000\184\000\184\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\255\255\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\062\000\059\000\059\000\059\000\059\000\059\000\059\000\
    \059\000\059\000\062\000\062\000\062\000\062\000\062\000\062\000\
    \062\000\062\000\062\000\062\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\062\000\062\000\062\000\062\000\062\000\
    \062\000\062\000\062\000\062\000\062\000\062\000\062\000\062\000\
    \062\000\062\000\062\000\062\000\062\000\062\000\062\000\062\000\
    \062\000\062\000\062\000\062\000\062\000\255\255\255\255\255\255\
    \255\255\062\000\255\255\062\000\062\000\062\000\062\000\062\000\
    \062\000\062\000\062\000\062\000\062\000\062\000\062\000\062\000\
    \062\000\062\000\062\000\062\000\062\000\062\000\062\000\062\000\
    \062\000\062\000\062\000\062\000\062\000\189\000\255\255\189\000\
    \189\000\189\000\189\000\189\000\189\000\189\000\189\000\189\000\
    \189\000\189\000\189\000\189\000\189\000\189\000\189\000\189\000\
    \189\000\189\000\189\000\189\000\189\000\189\000\189\000\189\000\
    \189\000\255\255\195\000\189\000\195\000\195\000\195\000\195\000\
    \195\000\195\000\195\000\195\000\195\000\195\000\195\000\195\000\
    \195\000\195\000\195\000\195\000\195\000\195\000\195\000\195\000\
    \195\000\195\000\195\000\195\000\195\000\195\000\255\255\255\255\
    \195\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\062\000\062\000\062\000\062\000\062\000\062\000\
    \062\000\062\000\062\000\062\000\062\000\062\000\062\000\062\000\
    \062\000\062\000\062\000\062\000\062\000\062\000\062\000\062\000\
    \062\000\255\255\062\000\062\000\062\000\062\000\062\000\062\000\
    \062\000\062\000\062\000\062\000\062\000\062\000\062\000\062\000\
    \062\000\062\000\062\000\062\000\062\000\062\000\062\000\062\000\
    \062\000\062\000\062\000\062\000\062\000\062\000\062\000\062\000\
    \062\000\255\255\062\000\062\000\062\000\062\000\062\000\062\000\
    \062\000\062\000\063\000\255\255\255\255\063\000\255\255\063\000\
    \063\000\255\255\255\255\063\000\063\000\255\255\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\255\255\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\255\255\255\255\255\255\063\000\
    \063\000\255\255\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\255\255\063\000\255\255\063\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \255\255\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \255\255\063\000\063\000\063\000\063\000\063\000\063\000\063\000\
    \063\000\064\000\255\255\255\255\064\000\064\000\064\000\255\255\
    \255\255\255\255\064\000\064\000\255\255\064\000\064\000\064\000\
    \255\255\081\000\255\255\081\000\255\255\255\255\255\255\255\255\
    \081\000\255\255\064\000\255\255\064\000\064\000\064\000\064\000\
    \064\000\081\000\081\000\081\000\081\000\081\000\081\000\081\000\
    \081\000\081\000\081\000\100\000\255\255\255\255\100\000\100\000\
    \100\000\255\255\255\255\255\255\100\000\100\000\255\255\100\000\
    \100\000\100\000\255\255\255\255\255\255\255\255\064\000\255\255\
    \255\255\255\255\255\255\255\255\100\000\255\255\100\000\100\000\
    \100\000\100\000\100\000\255\255\255\255\081\000\255\255\255\255\
    \255\255\255\255\255\255\081\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\064\000\255\255\064\000\081\000\
    \255\255\255\255\255\255\081\000\255\255\081\000\255\255\255\255\
    \100\000\081\000\255\255\255\255\255\255\255\255\102\000\255\255\
    \102\000\102\000\102\000\102\000\255\255\255\255\255\255\102\000\
    \102\000\255\255\102\000\102\000\102\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\100\000\102\000\
    \100\000\102\000\102\000\102\000\102\000\102\000\255\255\255\255\
    \255\255\113\000\255\255\255\255\113\000\113\000\113\000\255\255\
    \255\255\255\255\113\000\113\000\255\255\113\000\113\000\113\000\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\113\000\102\000\113\000\113\000\113\000\113\000\
    \113\000\255\255\255\255\255\255\255\255\255\255\114\000\255\255\
    \255\255\114\000\114\000\114\000\255\255\255\255\255\255\114\000\
    \114\000\255\255\114\000\114\000\114\000\255\255\255\255\255\255\
    \255\255\102\000\255\255\102\000\255\255\255\255\113\000\114\000\
    \255\255\114\000\114\000\114\000\114\000\114\000\255\255\255\255\
    \255\255\115\000\255\255\255\255\115\000\115\000\115\000\255\255\
    \255\255\255\255\115\000\115\000\255\255\115\000\115\000\115\000\
    \255\255\255\255\255\255\255\255\113\000\255\255\113\000\255\255\
    \255\255\081\000\115\000\114\000\115\000\115\000\115\000\115\000\
    \115\000\255\255\255\255\255\255\116\000\255\255\255\255\116\000\
    \116\000\116\000\255\255\255\255\255\255\116\000\116\000\255\255\
    \116\000\116\000\116\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\114\000\255\255\114\000\255\255\116\000\115\000\116\000\
    \116\000\116\000\116\000\116\000\255\255\255\255\255\255\121\000\
    \255\255\255\255\121\000\121\000\121\000\255\255\255\255\255\255\
    \121\000\121\000\255\255\121\000\121\000\121\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\115\000\255\255\115\000\255\255\
    \121\000\116\000\121\000\121\000\121\000\121\000\121\000\255\255\
    \255\255\255\255\131\000\255\255\255\255\131\000\131\000\131\000\
    \255\255\255\255\255\255\131\000\131\000\255\255\131\000\131\000\
    \131\000\255\255\255\255\255\255\255\255\255\255\255\255\116\000\
    \255\255\116\000\255\255\131\000\121\000\131\000\131\000\131\000\
    \131\000\131\000\255\255\255\255\255\255\132\000\255\255\255\255\
    \132\000\132\000\132\000\255\255\255\255\255\255\132\000\132\000\
    \255\255\132\000\132\000\132\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\121\000\255\255\121\000\255\255\132\000\131\000\
    \132\000\132\000\132\000\132\000\132\000\255\255\255\255\255\255\
    \133\000\255\255\255\255\133\000\133\000\133\000\255\255\255\255\
    \255\255\133\000\133\000\255\255\133\000\133\000\133\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\131\000\255\255\131\000\
    \255\255\133\000\132\000\133\000\133\000\133\000\133\000\133\000\
    \255\255\255\255\255\255\134\000\255\255\255\255\134\000\134\000\
    \134\000\255\255\255\255\255\255\134\000\134\000\255\255\134\000\
    \134\000\134\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \132\000\255\255\132\000\255\255\134\000\133\000\134\000\134\000\
    \134\000\134\000\134\000\255\255\255\255\255\255\135\000\255\255\
    \255\255\135\000\135\000\135\000\255\255\255\255\255\255\135\000\
    \135\000\255\255\135\000\135\000\135\000\255\255\255\255\150\000\
    \255\255\150\000\255\255\133\000\255\255\133\000\150\000\135\000\
    \134\000\135\000\135\000\135\000\135\000\135\000\255\255\150\000\
    \150\000\150\000\150\000\150\000\150\000\150\000\150\000\150\000\
    \150\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\134\000\255\255\
    \134\000\255\255\255\255\135\000\255\255\255\255\255\255\255\255\
    \255\255\255\255\173\000\255\255\255\255\173\000\255\255\255\255\
    \255\255\255\255\255\255\150\000\255\255\255\255\255\255\255\255\
    \255\255\150\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \173\000\135\000\173\000\135\000\255\255\150\000\255\255\173\000\
    \255\255\150\000\255\255\150\000\255\255\255\255\255\255\150\000\
    \173\000\173\000\173\000\173\000\173\000\173\000\173\000\173\000\
    \173\000\173\000\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\173\000\255\255\255\255\255\255\
    \255\255\255\255\173\000\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\173\000\255\255\
    \255\255\255\255\173\000\255\255\173\000\255\255\255\255\255\255\
    \173\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \173\000";
  Lexing.lex_base_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\010\000\
    \036\000\012\000\000\000\000\000\000\000\002\000\000\000\027\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\001\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\002\000\004\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000";
  Lexing.lex_backtrk_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\039\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000";
  Lexing.lex_default_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\019\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000";
  Lexing.lex_trans_code = 
   "\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\001\000\000\000\036\000\036\000\000\000\036\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \001\000\000\000\000\000\001\000\022\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\007\000\001\000\000\000\000\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\004\000\001\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\004\000\004\000\004\000\004\000\
    \004\000\004\000\004\000\004\000\004\000\004\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\000\
    \000\000\000\000\000\000\000\000\000\000";
  Lexing.lex_check_code = 
   "\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\022\000\108\000\173\000\180\000\108\000\181\000\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \022\000\255\255\108\000\000\000\109\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\103\000\104\000\255\255\255\255\
    \022\000\022\000\022\000\022\000\022\000\022\000\022\000\022\000\
    \022\000\022\000\103\000\103\000\103\000\103\000\103\000\103\000\
    \103\000\103\000\103\000\103\000\104\000\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\104\000\104\000\104\000\104\000\
    \104\000\104\000\104\000\104\000\104\000\104\000\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \108\000\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\255\
    \255\255\255\255\255\255\255\255\255\255";
  Lexing.lex_code = 
   "\255\004\255\255\005\255\255\007\255\006\255\255\003\255\000\004\
    \001\005\255\007\255\255\006\255\007\255\255\000\004\001\005\003\
    \006\002\007\255\001\255\255\000\001\255";
}

let rec token state lexbuf =
  lexbuf.Lexing.lex_mem <- Array.make 8 (-1) ;   __ocaml_lex_token_rec state lexbuf 0
and __ocaml_lex_token_rec state lexbuf __ocaml_lex_state =
  match Lexing.new_engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 308 "src/ocaml/preprocess/lexer_raw.mll"
                 (
      match state.preprocessor with
      | None ->
        fail (Illegal_character (Lexing.lexeme_char lexbuf 0))
              (Location.curr lexbuf)
      | Some _ ->
        update_loc lexbuf None 1 false 0;
        token state lexbuf )
# 1614 "src/ocaml/lexer_raw.ml"

  | 1 ->
# 317 "src/ocaml/preprocess/lexer_raw.mll"
      ( update_loc lexbuf None 1 false 0;
        match state.preprocessor with
        | None -> token state lexbuf
        | Some _ -> return EOL
      )
# 1623 "src/ocaml/lexer_raw.ml"

  | 2 ->
# 323 "src/ocaml/preprocess/lexer_raw.mll"
      ( token state lexbuf )
# 1628 "src/ocaml/lexer_raw.ml"

  | 3 ->
# 325 "src/ocaml/preprocess/lexer_raw.mll"
      ( return DOTLESS )
# 1633 "src/ocaml/lexer_raw.ml"

  | 4 ->
# 327 "src/ocaml/preprocess/lexer_raw.mll"
      ( return (keyword_or state (Lexing.lexeme lexbuf) (INFIXOP0 ">.")) )
# 1638 "src/ocaml/lexer_raw.ml"

  | 5 ->
# 329 "src/ocaml/preprocess/lexer_raw.mll"
      ( return (keyword_or state (Lexing.lexeme lexbuf) DOTTILDE) )
# 1643 "src/ocaml/lexer_raw.ml"

  | 6 ->
# 331 "src/ocaml/preprocess/lexer_raw.mll"
      ( return UNDERSCORE )
# 1648 "src/ocaml/lexer_raw.ml"

  | 7 ->
# 333 "src/ocaml/preprocess/lexer_raw.mll"
      ( return TILDE )
# 1653 "src/ocaml/lexer_raw.ml"

  | 8 ->
# 335 "src/ocaml/preprocess/lexer_raw.mll"
      ( lABEL (get_label_name lexbuf) )
# 1658 "src/ocaml/lexer_raw.ml"

  | 9 ->
# 337 "src/ocaml/preprocess/lexer_raw.mll"
      ( warn_latin1 lexbuf; lABEL (get_label_name lexbuf) )
# 1663 "src/ocaml/lexer_raw.ml"

  | 10 ->
# 339 "src/ocaml/preprocess/lexer_raw.mll"
      ( return QUESTION )
# 1668 "src/ocaml/lexer_raw.ml"

  | 11 ->
# 341 "src/ocaml/preprocess/lexer_raw.mll"
      ( oPTLABEL (get_label_name lexbuf) )
# 1673 "src/ocaml/lexer_raw.ml"

  | 12 ->
# 343 "src/ocaml/preprocess/lexer_raw.mll"
      ( warn_latin1 lexbuf; oPTLABEL (get_label_name lexbuf) )
# 1678 "src/ocaml/lexer_raw.ml"

  | 13 ->
# 345 "src/ocaml/preprocess/lexer_raw.mll"
      ( return QUESTIONQUESTION )
# 1683 "src/ocaml/lexer_raw.ml"

  | 14 ->
# 347 "src/ocaml/preprocess/lexer_raw.mll"
    ( let s = Lexing.lexeme lexbuf in
      return (try Hashtbl.find state.keywords s
              with Not_found ->
              try Hashtbl.find keyword_table s
              with Not_found ->
                LIDENT s) )
# 1693 "src/ocaml/lexer_raw.ml"

  | 15 ->
# 354 "src/ocaml/preprocess/lexer_raw.mll"
      ( warn_latin1 lexbuf; return (LIDENT (Lexing.lexeme lexbuf)) )
# 1698 "src/ocaml/lexer_raw.ml"

  | 16 ->
# 356 "src/ocaml/preprocess/lexer_raw.mll"
    ( (* Capitalized keywords for OUnit *)
      let s = Lexing.lexeme lexbuf in
      return (try Hashtbl.find state.keywords s
              with Not_found ->
              try Hashtbl.find keyword_table s
              with Not_found ->
                UIDENT s) )
# 1709 "src/ocaml/lexer_raw.ml"

  | 17 ->
# 364 "src/ocaml/preprocess/lexer_raw.mll"
      ( try
          return (INT (cvt_int_literal (Lexing.lexeme lexbuf)))
        with Failure _ ->
          fail (Literal_overflow "int") (Location.curr lexbuf)
      )
# 1718 "src/ocaml/lexer_raw.ml"

  | 18 ->
# 370 "src/ocaml/preprocess/lexer_raw.mll"
      ( return (FLOAT (remove_underscores(Lexing.lexeme lexbuf))) )
# 1723 "src/ocaml/lexer_raw.ml"

  | 19 ->
# 372 "src/ocaml/preprocess/lexer_raw.mll"
      ( try
          return (INT32 (cvt_int32_literal (Lexing.lexeme lexbuf)))
        with Failure _ ->
          fail (Literal_overflow "int32") (Location.curr lexbuf) )
# 1731 "src/ocaml/lexer_raw.ml"

  | 20 ->
# 377 "src/ocaml/preprocess/lexer_raw.mll"
      ( try
          return (INT64 (cvt_int64_literal (Lexing.lexeme lexbuf)))
        with Failure _ ->
          fail (Literal_overflow "int64") (Location.curr lexbuf) )
# 1739 "src/ocaml/lexer_raw.ml"

  | 21 ->
# 382 "src/ocaml/preprocess/lexer_raw.mll"
      ( try
          return (NATIVEINT (cvt_nativeint_literal (Lexing.lexeme lexbuf)))
        with Failure _ ->
          fail (Literal_overflow "nativeint") (Location.curr lexbuf) )
# 1747 "src/ocaml/lexer_raw.ml"

  | 22 ->
# 387 "src/ocaml/preprocess/lexer_raw.mll"
      ( Buffer.reset state.buffer;
        state.string_start_loc <- Location.curr lexbuf;
        string state lexbuf >>= fun () ->
        lexbuf.lex_start_p <- state.string_start_loc.Location.loc_start;
        state.string_start_loc <- Location.none;
        return (STRING (Buffer.contents state.buffer, None))
      )
# 1758 "src/ocaml/lexer_raw.ml"

  | 23 ->
# 395 "src/ocaml/preprocess/lexer_raw.mll"
      ( Buffer.reset state.buffer;
        let delim = Lexing.lexeme lexbuf in
        let delim = String.sub delim 1 (String.length delim - 2) in
        state.string_start_loc <- Location.curr lexbuf;
        quoted_string state delim lexbuf >>= fun () ->
        lexbuf.lex_start_p <- state.string_start_loc.Location.loc_start;
        state.string_start_loc <- Location.none;
        return (STRING (Buffer.contents state.buffer, Some delim)) )
# 1770 "src/ocaml/lexer_raw.ml"

  | 24 ->
# 404 "src/ocaml/preprocess/lexer_raw.mll"
    ( update_loc lexbuf None 1 false 1;
      return (CHAR (Lexing.lexeme_char lexbuf 1)) )
# 1776 "src/ocaml/lexer_raw.ml"

  | 25 ->
# 407 "src/ocaml/preprocess/lexer_raw.mll"
    ( return (CHAR (Lexing.lexeme_char lexbuf 1)) )
# 1781 "src/ocaml/lexer_raw.ml"

  | 26 ->
# 409 "src/ocaml/preprocess/lexer_raw.mll"
    ( return (CHAR (char_for_backslash (Lexing.lexeme_char lexbuf 2))) )
# 1786 "src/ocaml/lexer_raw.ml"

  | 27 ->
# 411 "src/ocaml/preprocess/lexer_raw.mll"
    ( char_for_decimal_code state lexbuf 2 >>= fun c -> return (CHAR c) )
# 1791 "src/ocaml/lexer_raw.ml"

  | 28 ->
# 413 "src/ocaml/preprocess/lexer_raw.mll"
    ( return (CHAR (char_for_hexadecimal_code lexbuf 3)) )
# 1796 "src/ocaml/lexer_raw.ml"

  | 29 ->
# 415 "src/ocaml/preprocess/lexer_raw.mll"
      ( let l = Lexing.lexeme lexbuf in
        let esc = String.sub l 1 (String.length l - 1) in
        fail (Illegal_escape esc) (Location.curr lexbuf)
      )
# 1804 "src/ocaml/lexer_raw.ml"

  | 30 ->
# 420 "src/ocaml/preprocess/lexer_raw.mll"
      ( let start_loc = Location.curr lexbuf in
        state.comment_start_loc <- [start_loc];
        Buffer.reset state.buffer;
        comment state lexbuf >>= fun end_loc ->
        let s = Buffer.contents state.buffer in
        Buffer.reset state.buffer;
        return (COMMENT (s, { start_loc with
                              Location.loc_end = end_loc.Location.loc_end }))
      )
# 1817 "src/ocaml/lexer_raw.ml"

  | 31 ->
# 430 "src/ocaml/preprocess/lexer_raw.mll"
      ( let loc = Location.curr lexbuf in
        Location.prerr_warning loc Warnings.Comment_start;
        state.comment_start_loc <- [loc];
        Buffer.reset state.buffer;
        comment state lexbuf >>= fun end_loc ->
        let s = Buffer.contents state.buffer in
        Buffer.reset state.buffer;
        return (COMMENT (s, { loc with Location.loc_end = end_loc.Location.loc_end }))
      )
# 1830 "src/ocaml/lexer_raw.ml"

  | 32 ->
# 440 "src/ocaml/preprocess/lexer_raw.mll"
      ( let loc = Location.curr lexbuf in
        Location.prerr_warning loc Warnings.Comment_not_end;
        lexbuf.Lexing.lex_curr_pos <- lexbuf.Lexing.lex_curr_pos - 1;
        let curpos = lexbuf.lex_curr_p in
        lexbuf.lex_curr_p <- { curpos with pos_cnum = curpos.pos_cnum - 1 };
        return STAR
      )
# 1841 "src/ocaml/lexer_raw.ml"

  | 33 ->
let
# 447 "src/ocaml/preprocess/lexer_raw.mll"
                                   num
# 1847 "src/ocaml/lexer_raw.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(0) lexbuf.Lexing.lex_mem.(1)
and
# 448 "src/ocaml/preprocess/lexer_raw.mll"
                                           name
# 1852 "src/ocaml/lexer_raw.ml"
= Lexing.sub_lexeme_opt lexbuf lexbuf.Lexing.lex_mem.(3) lexbuf.Lexing.lex_mem.(2) in
# 450 "src/ocaml/preprocess/lexer_raw.mll"
      ( update_loc lexbuf name (int_of_string num) true 0;
        token state lexbuf
      )
# 1858 "src/ocaml/lexer_raw.ml"

  | 34 ->
# 453 "src/ocaml/preprocess/lexer_raw.mll"
         ( return SHARP )
# 1863 "src/ocaml/lexer_raw.ml"

  | 35 ->
# 454 "src/ocaml/preprocess/lexer_raw.mll"
         ( return AMPERSAND )
# 1868 "src/ocaml/lexer_raw.ml"

  | 36 ->
# 455 "src/ocaml/preprocess/lexer_raw.mll"
         ( return AMPERAMPER )
# 1873 "src/ocaml/lexer_raw.ml"

  | 37 ->
# 456 "src/ocaml/preprocess/lexer_raw.mll"
         ( return BACKQUOTE )
# 1878 "src/ocaml/lexer_raw.ml"

  | 38 ->
# 457 "src/ocaml/preprocess/lexer_raw.mll"
         ( return QUOTE )
# 1883 "src/ocaml/lexer_raw.ml"

  | 39 ->
# 458 "src/ocaml/preprocess/lexer_raw.mll"
         ( return LPAREN )
# 1888 "src/ocaml/lexer_raw.ml"

  | 40 ->
# 459 "src/ocaml/preprocess/lexer_raw.mll"
         ( return RPAREN )
# 1893 "src/ocaml/lexer_raw.ml"

  | 41 ->
# 460 "src/ocaml/preprocess/lexer_raw.mll"
         ( return STAR )
# 1898 "src/ocaml/lexer_raw.ml"

  | 42 ->
# 461 "src/ocaml/preprocess/lexer_raw.mll"
         ( return COMMA )
# 1903 "src/ocaml/lexer_raw.ml"

  | 43 ->
# 462 "src/ocaml/preprocess/lexer_raw.mll"
         ( return MINUSGREATER )
# 1908 "src/ocaml/lexer_raw.ml"

  | 44 ->
# 463 "src/ocaml/preprocess/lexer_raw.mll"
         ( return DOT )
# 1913 "src/ocaml/lexer_raw.ml"

  | 45 ->
# 464 "src/ocaml/preprocess/lexer_raw.mll"
         ( return DOTDOT )
# 1918 "src/ocaml/lexer_raw.ml"

  | 46 ->
# 465 "src/ocaml/preprocess/lexer_raw.mll"
         ( return COLON )
# 1923 "src/ocaml/lexer_raw.ml"

  | 47 ->
# 466 "src/ocaml/preprocess/lexer_raw.mll"
         ( return COLONCOLON )
# 1928 "src/ocaml/lexer_raw.ml"

  | 48 ->
# 467 "src/ocaml/preprocess/lexer_raw.mll"
         ( return COLONEQUAL )
# 1933 "src/ocaml/lexer_raw.ml"

  | 49 ->
# 468 "src/ocaml/preprocess/lexer_raw.mll"
         ( return COLONGREATER )
# 1938 "src/ocaml/lexer_raw.ml"

  | 50 ->
# 469 "src/ocaml/preprocess/lexer_raw.mll"
         ( return SEMI )
# 1943 "src/ocaml/lexer_raw.ml"

  | 51 ->
# 470 "src/ocaml/preprocess/lexer_raw.mll"
         ( return SEMISEMI )
# 1948 "src/ocaml/lexer_raw.ml"

  | 52 ->
# 471 "src/ocaml/preprocess/lexer_raw.mll"
         ( return LESS )
# 1953 "src/ocaml/lexer_raw.ml"

  | 53 ->
# 472 "src/ocaml/preprocess/lexer_raw.mll"
         ( return LESSMINUS )
# 1958 "src/ocaml/lexer_raw.ml"

  | 54 ->
# 473 "src/ocaml/preprocess/lexer_raw.mll"
         ( return EQUAL )
# 1963 "src/ocaml/lexer_raw.ml"

  | 55 ->
# 474 "src/ocaml/preprocess/lexer_raw.mll"
         ( return LBRACKET )
# 1968 "src/ocaml/lexer_raw.ml"

  | 56 ->
# 475 "src/ocaml/preprocess/lexer_raw.mll"
         ( return LBRACKETBAR )
# 1973 "src/ocaml/lexer_raw.ml"

  | 57 ->
# 476 "src/ocaml/preprocess/lexer_raw.mll"
         ( return LBRACKETLESS )
# 1978 "src/ocaml/lexer_raw.ml"

  | 58 ->
# 477 "src/ocaml/preprocess/lexer_raw.mll"
         ( return LBRACKETGREATER )
# 1983 "src/ocaml/lexer_raw.ml"

  | 59 ->
# 478 "src/ocaml/preprocess/lexer_raw.mll"
         ( return RBRACKET )
# 1988 "src/ocaml/lexer_raw.ml"

  | 60 ->
# 479 "src/ocaml/preprocess/lexer_raw.mll"
         ( return LBRACE )
# 1993 "src/ocaml/lexer_raw.ml"

  | 61 ->
# 480 "src/ocaml/preprocess/lexer_raw.mll"
         ( return LBRACELESS )
# 1998 "src/ocaml/lexer_raw.ml"

  | 62 ->
# 481 "src/ocaml/preprocess/lexer_raw.mll"
         ( return BAR )
# 2003 "src/ocaml/lexer_raw.ml"

  | 63 ->
# 482 "src/ocaml/preprocess/lexer_raw.mll"
         ( return BARBAR )
# 2008 "src/ocaml/lexer_raw.ml"

  | 64 ->
# 483 "src/ocaml/preprocess/lexer_raw.mll"
         ( return BARRBRACKET )
# 2013 "src/ocaml/lexer_raw.ml"

  | 65 ->
# 484 "src/ocaml/preprocess/lexer_raw.mll"
         ( return GREATER )
# 2018 "src/ocaml/lexer_raw.ml"

  | 66 ->
# 485 "src/ocaml/preprocess/lexer_raw.mll"
         ( return GREATERRBRACKET )
# 2023 "src/ocaml/lexer_raw.ml"

  | 67 ->
# 486 "src/ocaml/preprocess/lexer_raw.mll"
         ( return RBRACE )
# 2028 "src/ocaml/lexer_raw.ml"

  | 68 ->
# 487 "src/ocaml/preprocess/lexer_raw.mll"
         ( return GREATERRBRACE )
# 2033 "src/ocaml/lexer_raw.ml"

  | 69 ->
# 488 "src/ocaml/preprocess/lexer_raw.mll"
         ( return LBRACKETAT )
# 2038 "src/ocaml/lexer_raw.ml"

  | 70 ->
# 489 "src/ocaml/preprocess/lexer_raw.mll"
         ( return LBRACKETPERCENT )
# 2043 "src/ocaml/lexer_raw.ml"

  | 71 ->
# 490 "src/ocaml/preprocess/lexer_raw.mll"
          ( return LBRACKETPERCENTPERCENT )
# 2048 "src/ocaml/lexer_raw.ml"

  | 72 ->
# 491 "src/ocaml/preprocess/lexer_raw.mll"
          ( return LBRACKETATAT )
# 2053 "src/ocaml/lexer_raw.ml"

  | 73 ->
# 492 "src/ocaml/preprocess/lexer_raw.mll"
           ( return LBRACKETATATAT )
# 2058 "src/ocaml/lexer_raw.ml"

  | 74 ->
# 493 "src/ocaml/preprocess/lexer_raw.mll"
         ( return BANG )
# 2063 "src/ocaml/lexer_raw.ml"

  | 75 ->
# 494 "src/ocaml/preprocess/lexer_raw.mll"
         ( return (INFIXOP0 "!=") )
# 2068 "src/ocaml/lexer_raw.ml"

  | 76 ->
# 495 "src/ocaml/preprocess/lexer_raw.mll"
         ( return PLUS )
# 2073 "src/ocaml/lexer_raw.ml"

  | 77 ->
# 496 "src/ocaml/preprocess/lexer_raw.mll"
         ( return PLUSDOT )
# 2078 "src/ocaml/lexer_raw.ml"

  | 78 ->
# 497 "src/ocaml/preprocess/lexer_raw.mll"
         ( return PLUSEQ )
# 2083 "src/ocaml/lexer_raw.ml"

  | 79 ->
# 498 "src/ocaml/preprocess/lexer_raw.mll"
         ( return MINUS )
# 2088 "src/ocaml/lexer_raw.ml"

  | 80 ->
# 499 "src/ocaml/preprocess/lexer_raw.mll"
         ( return MINUSDOT )
# 2093 "src/ocaml/lexer_raw.ml"

  | 81 ->
# 502 "src/ocaml/preprocess/lexer_raw.mll"
            ( return (PREFIXOP(Lexing.lexeme lexbuf)) )
# 2098 "src/ocaml/lexer_raw.ml"

  | 82 ->
# 504 "src/ocaml/preprocess/lexer_raw.mll"
            ( return (PREFIXOP(Lexing.lexeme lexbuf)) )
# 2103 "src/ocaml/lexer_raw.ml"

  | 83 ->
# 506 "src/ocaml/preprocess/lexer_raw.mll"
            ( return (keyword_or state (Lexing.lexeme lexbuf)
                       (INFIXOP0(Lexing.lexeme lexbuf))) )
# 2109 "src/ocaml/lexer_raw.ml"

  | 84 ->
# 509 "src/ocaml/preprocess/lexer_raw.mll"
            ( return (INFIXOP1(Lexing.lexeme lexbuf)) )
# 2114 "src/ocaml/lexer_raw.ml"

  | 85 ->
# 511 "src/ocaml/preprocess/lexer_raw.mll"
            ( return (INFIXOP2(Lexing.lexeme lexbuf)) )
# 2119 "src/ocaml/lexer_raw.ml"

  | 86 ->
# 513 "src/ocaml/preprocess/lexer_raw.mll"
            ( return (INFIXOP4(Lexing.lexeme lexbuf)) )
# 2124 "src/ocaml/lexer_raw.ml"

  | 87 ->
# 514 "src/ocaml/preprocess/lexer_raw.mll"
            ( return PERCENT )
# 2129 "src/ocaml/lexer_raw.ml"

  | 88 ->
# 516 "src/ocaml/preprocess/lexer_raw.mll"
            ( return (INFIXOP3(Lexing.lexeme lexbuf)) )
# 2134 "src/ocaml/lexer_raw.ml"

  | 89 ->
# 518 "src/ocaml/preprocess/lexer_raw.mll"
            ( let s = Lexing.lexeme lexbuf in
              return (try Hashtbl.find state.keywords s
                      with Not_found -> SHARPOP s) )
# 2141 "src/ocaml/lexer_raw.ml"

  | 90 ->
# 522 "src/ocaml/preprocess/lexer_raw.mll"
            ( return (LETOP(Lexing.lexeme lexbuf)) )
# 2146 "src/ocaml/lexer_raw.ml"

  | 91 ->
# 525 "src/ocaml/preprocess/lexer_raw.mll"
            ( let s = Lexing.lexeme lexbuf in
              return (try Hashtbl.find state.keywords s
                      with Not_found -> SHARPOP s) )
# 2153 "src/ocaml/lexer_raw.ml"

  | 92 ->
# 528 "src/ocaml/preprocess/lexer_raw.mll"
        ( return EOF )
# 2158 "src/ocaml/lexer_raw.ml"

  | 93 ->
# 531 "src/ocaml/preprocess/lexer_raw.mll"
      ( fail (Illegal_character (Lexing.lexeme_char lexbuf 0))
              (Location.curr lexbuf)
      )
# 2165 "src/ocaml/lexer_raw.ml"

  | __ocaml_lex_state -> __ocaml_lex_refill 
      (fun lexbuf -> lexbuf.Lexing.refill_buff lexbuf; 
         __ocaml_lex_token_rec state lexbuf __ocaml_lex_state) lexbuf

and comment state lexbuf =
    __ocaml_lex_comment_rec state lexbuf 136
and __ocaml_lex_comment_rec state lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 537 "src/ocaml/preprocess/lexer_raw.mll"
      ( state.comment_start_loc <- (Location.curr lexbuf) :: state.comment_start_loc;
      Buffer.add_string state.buffer (Lexing.lexeme lexbuf);
      comment state lexbuf
    )
# 2181 "src/ocaml/lexer_raw.ml"

  | 1 ->
# 542 "src/ocaml/preprocess/lexer_raw.mll"
      ( match state.comment_start_loc with
        | [] -> assert false
        | [_] -> state.comment_start_loc <- []; return (Location.curr lexbuf)
        | _ :: l -> state.comment_start_loc <- l;
                  Buffer.add_string state.buffer (Lexing.lexeme lexbuf);
                  comment state lexbuf
       )
# 2192 "src/ocaml/lexer_raw.ml"

  | 2 ->
# 550 "src/ocaml/preprocess/lexer_raw.mll"
      (
        state.string_start_loc <- Location.curr lexbuf;
        Buffer.add_char state.buffer '\"';
        let buffer = state.buffer in
        state.buffer <- Buffer.create 15;
        (catch (string state lexbuf) (fun e l -> match e with
             | Unterminated_string ->
               begin match state.comment_start_loc with
                 | [] -> assert false
                 | loc :: _ ->
                   let start = List.hd (List.rev state.comment_start_loc) in
                   state.comment_start_loc <- [];
                   fail (Unterminated_string_in_comment (start, l)) loc
               end
             | e -> fail e l
           )
        ) >>= fun () ->
      state.string_start_loc <- Location.none;
      Buffer.add_string buffer (String.escaped (Buffer.contents state.buffer));
      state.buffer <- buffer;
      Buffer.add_char state.buffer '\"';
      comment state lexbuf )
# 2218 "src/ocaml/lexer_raw.ml"

  | 3 ->
# 573 "src/ocaml/preprocess/lexer_raw.mll"
      (
        let delim = Lexing.lexeme lexbuf in
        let delim = String.sub delim 1 (String.length delim - 2) in
        state.string_start_loc <- Location.curr lexbuf;
        Buffer.add_string state.buffer (Lexing.lexeme lexbuf);
        (catch (quoted_string state delim lexbuf) (fun e l -> match e with
             | Unterminated_string ->
               begin match state.comment_start_loc with
                 | [] -> assert false
                 | loc :: _ ->
                   let start = List.hd (List.rev state.comment_start_loc) in
                   state.comment_start_loc <- [];
                   fail (Unterminated_string_in_comment (start, l)) loc
               end
             | e -> fail e l
           )
        ) >>= fun () ->
        state.string_start_loc <- Location.none;
        Buffer.add_char state.buffer '|';
        Buffer.add_string state.buffer delim;
        Buffer.add_char state.buffer '}';
        comment state lexbuf )
# 2244 "src/ocaml/lexer_raw.ml"

  | 4 ->
# 597 "src/ocaml/preprocess/lexer_raw.mll"
      ( Buffer.add_string state.buffer (Lexing.lexeme lexbuf); comment state lexbuf )
# 2249 "src/ocaml/lexer_raw.ml"

  | 5 ->
# 599 "src/ocaml/preprocess/lexer_raw.mll"
      ( update_loc lexbuf None 1 false 1;
        Buffer.add_string state.buffer (Lexing.lexeme lexbuf);
        comment state lexbuf
      )
# 2257 "src/ocaml/lexer_raw.ml"

  | 6 ->
# 604 "src/ocaml/preprocess/lexer_raw.mll"
      ( Buffer.add_string state.buffer (Lexing.lexeme lexbuf); comment state lexbuf )
# 2262 "src/ocaml/lexer_raw.ml"

  | 7 ->
# 606 "src/ocaml/preprocess/lexer_raw.mll"
      ( Buffer.add_string state.buffer (Lexing.lexeme lexbuf); comment state lexbuf )
# 2267 "src/ocaml/lexer_raw.ml"

  | 8 ->
# 608 "src/ocaml/preprocess/lexer_raw.mll"
      ( Buffer.add_string state.buffer (Lexing.lexeme lexbuf); comment state lexbuf )
# 2272 "src/ocaml/lexer_raw.ml"

  | 9 ->
# 610 "src/ocaml/preprocess/lexer_raw.mll"
      ( Buffer.add_string state.buffer (Lexing.lexeme lexbuf); comment state lexbuf )
# 2277 "src/ocaml/lexer_raw.ml"

  | 10 ->
# 612 "src/ocaml/preprocess/lexer_raw.mll"
      ( match state.comment_start_loc with
        | [] -> assert false
        | loc :: _ ->
          let start = List.hd (List.rev state.comment_start_loc) in
          state.comment_start_loc <- [];
          fail (Unterminated_comment start) loc
      )
# 2288 "src/ocaml/lexer_raw.ml"

  | 11 ->
# 620 "src/ocaml/preprocess/lexer_raw.mll"
      ( update_loc lexbuf None 1 false 0;
        Buffer.add_string state.buffer (Lexing.lexeme lexbuf);
        comment state lexbuf
      )
# 2296 "src/ocaml/lexer_raw.ml"

  | 12 ->
# 625 "src/ocaml/preprocess/lexer_raw.mll"
      ( Buffer.add_string state.buffer (Lexing.lexeme lexbuf); comment state lexbuf )
# 2301 "src/ocaml/lexer_raw.ml"

  | __ocaml_lex_state -> __ocaml_lex_refill 
      (fun lexbuf -> lexbuf.Lexing.refill_buff lexbuf; 
         __ocaml_lex_comment_rec state lexbuf __ocaml_lex_state) lexbuf

and string state lexbuf =
  lexbuf.Lexing.lex_mem <- Array.make 2 (-1) ;   __ocaml_lex_string_rec state lexbuf 168
and __ocaml_lex_string_rec state lexbuf __ocaml_lex_state =
  match Lexing.new_engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 629 "src/ocaml/preprocess/lexer_raw.mll"
      ( return () )
# 2314 "src/ocaml/lexer_raw.ml"

  | 1 ->
let
# 630 "src/ocaml/preprocess/lexer_raw.mll"
                                  space
# 2320 "src/ocaml/lexer_raw.ml"
= Lexing.sub_lexeme lexbuf lexbuf.Lexing.lex_mem.(0) lexbuf.Lexing.lex_curr_pos in
# 631 "src/ocaml/preprocess/lexer_raw.mll"
      ( update_loc lexbuf None 1 false (String.length space);
        string state lexbuf
      )
# 2326 "src/ocaml/lexer_raw.ml"

  | 2 ->
# 635 "src/ocaml/preprocess/lexer_raw.mll"
      ( Buffer.add_char state.buffer
          (char_for_backslash (Lexing.lexeme_char lexbuf 1));
        string state lexbuf )
# 2333 "src/ocaml/lexer_raw.ml"

  | 3 ->
# 639 "src/ocaml/preprocess/lexer_raw.mll"
      ( char_for_decimal_code state lexbuf 1 >>= fun c ->
        Buffer.add_char state.buffer c;
        string state lexbuf )
# 2340 "src/ocaml/lexer_raw.ml"

  | 4 ->
# 643 "src/ocaml/preprocess/lexer_raw.mll"
      ( Buffer.add_char state.buffer (char_for_hexadecimal_code lexbuf 2);
        string state lexbuf )
# 2346 "src/ocaml/lexer_raw.ml"

  | 5 ->
# 646 "src/ocaml/preprocess/lexer_raw.mll"
      ( if in_comment state
        then string state lexbuf
        else begin
(*  Should be an error, but we are very lax.
                  fail (Illegal_escape (Lexing.lexeme lexbuf),
                        (Location.curr lexbuf)
*)
          let loc = Location.curr lexbuf in
          Location.prerr_warning loc Warnings.Illegal_backslash;
          Buffer.add_char state.buffer (Lexing.lexeme_char lexbuf 0);
          Buffer.add_char state.buffer (Lexing.lexeme_char lexbuf 1);
          string state lexbuf
        end
      )
# 2364 "src/ocaml/lexer_raw.ml"

  | 6 ->
# 661 "src/ocaml/preprocess/lexer_raw.mll"
      ( if not (in_comment state) then
          Location.prerr_warning (Location.curr lexbuf) Warnings.Eol_in_string;
        update_loc lexbuf None 1 false 0;
        Buffer.add_string state.buffer (Lexing.lexeme lexbuf);
        string state lexbuf
      )
# 2374 "src/ocaml/lexer_raw.ml"

  | 7 ->
# 668 "src/ocaml/preprocess/lexer_raw.mll"
      ( let loc = state.string_start_loc in
        state.string_start_loc <- Location.none;
        fail Unterminated_string loc )
# 2381 "src/ocaml/lexer_raw.ml"

  | 8 ->
# 672 "src/ocaml/preprocess/lexer_raw.mll"
      ( Buffer.add_char state.buffer (Lexing.lexeme_char lexbuf 0);
        string state lexbuf )
# 2387 "src/ocaml/lexer_raw.ml"

  | __ocaml_lex_state -> __ocaml_lex_refill 
      (fun lexbuf -> lexbuf.Lexing.refill_buff lexbuf; 
         __ocaml_lex_string_rec state lexbuf __ocaml_lex_state) lexbuf

and quoted_string state delim lexbuf =
    __ocaml_lex_quoted_string_rec state delim lexbuf 187
and __ocaml_lex_quoted_string_rec state delim lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 677 "src/ocaml/preprocess/lexer_raw.mll"
      ( update_loc lexbuf None 1 false 0;
        Buffer.add_string state.buffer (Lexing.lexeme lexbuf);
        quoted_string state delim lexbuf
      )
# 2403 "src/ocaml/lexer_raw.ml"

  | 1 ->
# 682 "src/ocaml/preprocess/lexer_raw.mll"
      ( let loc = state.string_start_loc in
        state.string_start_loc <- Location.none;
        fail Unterminated_string loc )
# 2410 "src/ocaml/lexer_raw.ml"

  | 2 ->
# 686 "src/ocaml/preprocess/lexer_raw.mll"
      (
        let edelim = Lexing.lexeme lexbuf in
        let edelim = String.sub edelim 1 (String.length edelim - 2) in
        if delim = edelim then return ()
        else (Buffer.add_string state.buffer (Lexing.lexeme lexbuf);
              quoted_string state delim lexbuf)
      )
# 2421 "src/ocaml/lexer_raw.ml"

  | 3 ->
# 694 "src/ocaml/preprocess/lexer_raw.mll"
      ( Buffer.add_char state.buffer (Lexing.lexeme_char lexbuf 0);
        quoted_string state delim lexbuf )
# 2427 "src/ocaml/lexer_raw.ml"

  | __ocaml_lex_state -> __ocaml_lex_refill 
      (fun lexbuf -> lexbuf.Lexing.refill_buff lexbuf; 
         __ocaml_lex_quoted_string_rec state delim lexbuf __ocaml_lex_state) lexbuf

and skip_sharp_bang state lexbuf =
    __ocaml_lex_skip_sharp_bang_rec state lexbuf 196
and __ocaml_lex_skip_sharp_bang_rec state lexbuf __ocaml_lex_state =
  match Lexing.engine __ocaml_lex_tables __ocaml_lex_state lexbuf with
      | 0 ->
# 699 "src/ocaml/preprocess/lexer_raw.mll"
      ( update_loc lexbuf None 3 false 0; token state lexbuf )
# 2440 "src/ocaml/lexer_raw.ml"

  | 1 ->
# 701 "src/ocaml/preprocess/lexer_raw.mll"
      ( update_loc lexbuf None 1 false 0; token state lexbuf )
# 2445 "src/ocaml/lexer_raw.ml"

  | 2 ->
# 702 "src/ocaml/preprocess/lexer_raw.mll"
       ( token state lexbuf )
# 2450 "src/ocaml/lexer_raw.ml"

  | __ocaml_lex_state -> __ocaml_lex_refill 
      (fun lexbuf -> lexbuf.Lexing.refill_buff lexbuf; 
         __ocaml_lex_skip_sharp_bang_rec state lexbuf __ocaml_lex_state) lexbuf

;;

# 704 "src/ocaml/preprocess/lexer_raw.mll"
 
  type comment = string * Location.t

  (* preprocessor support not implemented, not compatible with monadic
     interface *)

  let rec token_without_comments state lexbuf =
    token state lexbuf >>= function
    | COMMENT (s, comment_loc) ->
      token_without_comments state lexbuf
    | tok -> return tok

# 2471 "src/ocaml/lexer_raw.ml"
